@@ -28,7 +28,7 @@ extern "C" {
 #include <errno.h>
 #include <stddef.h>
 #if defined _WIN32
-#include "winsock2.h"
+#include <winsock2.h>
 #endif
 
 /*  Handle DSO symbol visibility                                             */
@@ -55,7 +55,7 @@ extern "C" {
 /*  Version macros for compile-time API version detection                     */
 #define ZMQ_VERSION_MAJOR 2
 #define ZMQ_VERSION_MINOR 1
-#define ZMQ_VERSION_PATCH 7
+#define ZMQ_VERSION_PATCH 9
 
 #define ZMQ_MAKE_VERSION(major, minor, patch) \
     ((major) * 10000 + (minor) * 100 + (patch))
