@@ -1,19 +1,19 @@
 #!/usr/bin/python
 
 import os
-import sys
+import platform
 import time
 
 build_date = time.strftime ("%Y-%m-%d")
 build_time = time.strftime ("%H:%M:%S")
-build_rev = os.popen('svnversion -n .').read();
+build_rev = filter (str.isdigit, "$Revision: 1350 $")
 
 print """
 /* vim:ts=8:sts=8:sw=4:noai:noexpandtab
  * 
  * OpenPGM version.
  *
- * Copyright (c) 2006-2009 Miru Limited.
+ * Copyright (c) 2006-2011 Miru Limited.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,23 +30,23 @@ print """
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <glib.h>
-
-#include "pgm/version.h"
+#include <impl/framework.h>
+#include <pgm/version.h>
 
 
 /* globals */
 
-const guint pgm_major_version = 2;
-const guint pgm_minor_version = 1;
-const guint pgm_micro_version = 28;
+const unsigned pgm_major_version = 5;
+const unsigned pgm_minor_version = 1;
+const unsigned pgm_micro_version = 115;
 const char* pgm_build_date = "%s";
 const char* pgm_build_time = "%s";
-const char* pgm_build_platform = "%s";
+const char* pgm_build_system = "%s";
+const char* pgm_build_machine = "%s";
 const char* pgm_build_revision = "%s";
 
 
 /* eof */
-"""%(build_date, build_time, sys.platform, build_rev)
+"""%(build_date, build_time, platform.system(), platform.machine(), build_rev)
 
 # end of file
