@@ -1,19 +1,20 @@
 /*
-    Copyright (c) 2007-2010 iMatix Corporation
+    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
     0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the Lesser GNU General Public License as published by
+    the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     0MQ is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    Lesser GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the Lesser GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
@@ -22,123 +23,170 @@
 #include "err.hpp"
 #include "platform.hpp"
 
+const char *zmq::errno_to_string (int errno_)
+{
+    switch (errno_) {
+#if defined ZMQ_HAVE_WINDOWS
+    case ENOTSUP:
+        return "Not supported";
+    case EPROTONOSUPPORT:
+        return "Protocol not supported";
+    case ENOBUFS:
+        return "No buffer space available";
+    case ENETDOWN:
+        return "Network is down";
+    case EADDRINUSE:
+        return "Address in use";
+    case EADDRNOTAVAIL:
+        return "Address not available";
+    case ECONNREFUSED:
+        return "Connection refused";
+    case EINPROGRESS:
+        return "Operation in progress";
+#endif
+    case EFSM:
+        return "Operation cannot be accomplished in current state";
+    case ENOCOMPATPROTO:
+        return "The protocol is not compatible with the socket type";
+    case ETERM:
+        return "Context was terminated";
+    case EMTHREAD:
+        return "No thread available";
+    default:
+#if defined _MSC_VER
+#pragma warning (push)
+#pragma warning (disable:4996)
+#endif
+        return strerror (errno_);
+#if defined _MSC_VER
+#pragma warning (pop)
+#endif
+    }
+}
+
 #ifdef ZMQ_HAVE_WINDOWS
 
 const char *zmq::wsa_error()
 {
-    int errcode = WSAGetLastError ();
+    int no = WSAGetLastError ();
     //  TODO: This is not a generic way to handle this...
-    if (errcode == WSAEWOULDBLOCK)
+    if (no == WSAEWOULDBLOCK)
         return NULL;
 
+    return wsa_error_no (no);
+}
+
+const char *zmq::wsa_error_no (int no_)
+{
     //  TODO:  It seems that list of Windows socket errors is longer than this.
     //         Investigate whether there's a way to convert it into the string
     //         automatically (wsaError->HRESULT->string?).
     return
-        (errcode == WSABASEERR) ?
+        (no_ == WSABASEERR) ?
             "No Error" : 
-        (errcode == WSAEINTR) ?
+        (no_ == WSAEINTR) ?
             "Interrupted system call" : 
-        (errcode == WSAEBADF) ?
+        (no_ == WSAEBADF) ?
             "Bad file number" : 
-        (errcode == WSAEACCES) ?
+        (no_ == WSAEACCES) ?
             "Permission denied" : 
-        (errcode == WSAEFAULT) ?
+        (no_ == WSAEFAULT) ?
             "Bad address" : 
-        (errcode == WSAEINVAL) ?
+        (no_ == WSAEINVAL) ?
             "Invalid argument" : 
-        (errcode == WSAEMFILE) ?
+        (no_ == WSAEMFILE) ?
             "Too many open files" : 
-        (errcode == WSAEWOULDBLOCK) ?
+        (no_ == WSAEWOULDBLOCK) ?
             "Operation would block" : 
-        (errcode == WSAEINPROGRESS) ?
+        (no_ == WSAEINPROGRESS) ?
             "Operation now in progress" : 
-        (errcode == WSAEALREADY) ?
+        (no_ == WSAEALREADY) ?
             "Operation already in progress" : 
-        (errcode == WSAENOTSOCK) ?
+        (no_ == WSAENOTSOCK) ?
             "Socket operation on non-socket" : 
-        (errcode == WSAEDESTADDRREQ) ?
+        (no_ == WSAEDESTADDRREQ) ?
             "Destination address required" : 
-        (errcode == WSAEMSGSIZE) ?
+        (no_ == WSAEMSGSIZE) ?
             "Message too long" : 
-        (errcode == WSAEPROTOTYPE) ?
+        (no_ == WSAEPROTOTYPE) ?
             "Protocol wrong type for socket" : 
-        (errcode == WSAENOPROTOOPT) ?
+        (no_ == WSAENOPROTOOPT) ?
             "Bad protocol option" : 
-        (errcode == WSAEPROTONOSUPPORT) ?
+        (no_ == WSAEPROTONOSUPPORT) ?
             "Protocol not supported" : 
-        (errcode == WSAESOCKTNOSUPPORT) ?
+        (no_ == WSAESOCKTNOSUPPORT) ?
             "Socket type not supported" : 
-        (errcode == WSAEOPNOTSUPP) ?
+        (no_ == WSAEOPNOTSUPP) ?
             "Operation not supported on socket" : 
-        (errcode == WSAEPFNOSUPPORT) ?
+        (no_ == WSAEPFNOSUPPORT) ?
             "Protocol family not supported" : 
-        (errcode == WSAEAFNOSUPPORT) ?
+        (no_ == WSAEAFNOSUPPORT) ?
             "Address family not supported by protocol family" : 
-        (errcode == WSAEADDRINUSE) ?
+        (no_ == WSAEADDRINUSE) ?
             "Address already in use" : 
-        (errcode == WSAEADDRNOTAVAIL) ?
+        (no_ == WSAEADDRNOTAVAIL) ?
             "Can't assign requested address" : 
-        (errcode == WSAENETDOWN) ?
+        (no_ == WSAENETDOWN) ?
             "Network is down" : 
-        (errcode == WSAENETUNREACH) ?
+        (no_ == WSAENETUNREACH) ?
             "Network is unreachable" : 
-        (errcode == WSAENETRESET) ?
+        (no_ == WSAENETRESET) ?
             "Net dropped connection or reset" : 
-        (errcode == WSAECONNABORTED) ?
+        (no_ == WSAECONNABORTED) ?
             "Software caused connection abort" : 
-        (errcode == WSAECONNRESET) ?
+        (no_ == WSAECONNRESET) ?
             "Connection reset by peer" : 
-        (errcode == WSAENOBUFS) ?
+        (no_ == WSAENOBUFS) ?
             "No buffer space available" : 
-        (errcode == WSAEISCONN) ?
+        (no_ == WSAEISCONN) ?
             "Socket is already connected" : 
-        (errcode == WSAENOTCONN) ?
+        (no_ == WSAENOTCONN) ?
             "Socket is not connected" : 
-        (errcode == WSAESHUTDOWN) ?
+        (no_ == WSAESHUTDOWN) ?
             "Can't send after socket shutdown" : 
-        (errcode == WSAETOOMANYREFS) ?
+        (no_ == WSAETOOMANYREFS) ?
             "Too many references can't splice" : 
-        (errcode == WSAETIMEDOUT) ?
+        (no_ == WSAETIMEDOUT) ?
             "Connection timed out" : 
-        (errcode == WSAECONNREFUSED) ?
+        (no_ == WSAECONNREFUSED) ?
             "Connection refused" : 
-        (errcode == WSAELOOP) ?
+        (no_ == WSAELOOP) ?
             "Too many levels of symbolic links" : 
-        (errcode == WSAENAMETOOLONG) ?
+        (no_ == WSAENAMETOOLONG) ?
             "File name too long" : 
-        (errcode == WSAEHOSTDOWN) ?
+        (no_ == WSAEHOSTDOWN) ?
             "Host is down" : 
-        (errcode == WSAEHOSTUNREACH) ?
+        (no_ == WSAEHOSTUNREACH) ?
             "No Route to Host" : 
-        (errcode == WSAENOTEMPTY) ?
+        (no_ == WSAENOTEMPTY) ?
             "Directory not empty" : 
-        (errcode == WSAEPROCLIM) ?
+        (no_ == WSAEPROCLIM) ?
             "Too many processes" : 
-        (errcode == WSAEUSERS) ?
+        (no_ == WSAEUSERS) ?
             "Too many users" : 
-        (errcode == WSAEDQUOT) ?
+        (no_ == WSAEDQUOT) ?
             "Disc Quota Exceeded" : 
-        (errcode == WSAESTALE) ?
+        (no_ == WSAESTALE) ?
             "Stale NFS file handle" : 
-        (errcode == WSAEREMOTE) ?
+        (no_ == WSAEREMOTE) ?
             "Too many levels of remote in path" : 
-        (errcode == WSASYSNOTREADY) ?
+        (no_ == WSASYSNOTREADY) ?
             "Network SubSystem is unavailable" : 
-        (errcode == WSAVERNOTSUPPORTED) ?
+        (no_ == WSAVERNOTSUPPORTED) ?
             "WINSOCK DLL Version out of range" : 
-        (errcode == WSANOTINITIALISED) ?
+        (no_ == WSANOTINITIALISED) ?
             "Successful WSASTARTUP not yet performed" : 
-        (errcode == WSAHOST_NOT_FOUND) ?
+        (no_ == WSAHOST_NOT_FOUND) ?
             "Host not found" : 
-        (errcode == WSATRY_AGAIN) ?
+        (no_ == WSATRY_AGAIN) ?
             "Non-Authoritative Host not found" : 
-        (errcode == WSANO_RECOVERY) ?
+        (no_ == WSANO_RECOVERY) ?
             "Non-Recoverable errors: FORMERR REFUSED NOTIMP" : 
-        (errcode == WSANO_DATA) ?
+        (no_ == WSANO_DATA) ?
             "Valid name no data record of requested" :
         "error not defined"; 
 }
+
 void zmq::win_error (char *buffer_, size_t buffer_size_)
 {
     DWORD errcode = GetLastError ();
